package consume

import (
	"applet/app/cfg"
	"applet/app/db"
	db2 "applet/app/db/gim"
	"applet/app/svc"
	utils2 "applet/app/utils"
	"applet/app/utils/logx"
	utils "applet/app/utils/rpc"
	"applet/consume/md"
	"applet/pkg/pb"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"strconv"
	"time"
)

func DouShenUserRegisterConsumeForOperationCenter(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>DouShenUserRegisterConsumeForOperationCenter>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
			err = handleDouShenUserRegisterConsumeForOperationCenter(res.Body)
			if err != nil {
				fmt.Println("!!!!!!!err!!!!!!!", err)
				_ = res.Reject(false)
			} else {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleDouShenUserRegisterConsumeForOperationCenter(msgData []byte) error {
	time.Sleep(time.Duration(6) * time.Second)
	//1、解析mq中queue的数据结构体
	var msg *md.DouShenUserRegisterMessageStructForOperationCenter
	err := json.Unmarshal(msgData, &msg)
	if err != nil {
		return err
	}
	//2、查找用户对应im系统中的数据
	gimUser, err := db2.UserGetOneByParams(db.ImDb, msg.MasterId, map[string]interface{}{
		"key":   "phone_number",
		"value": msg.Phone,
	})
	if err != nil {
		return err
	}

	if gimUser == nil {
		time.Sleep(time.Duration(10) * time.Second)
		gimUser, err := db2.UserGetOneByParams(db.ImDb, msg.MasterId, map[string]interface{}{
			"key":   "phone_number",
			"value": msg.Phone,
		})
		if err != nil {
			return err
		}
		if gimUser == nil {
			return errors.New("用户暂未注册im系统")
		}
	}

	//2、查找是否有群
	OperationGroup, err := db.DouShenImGroupGetOneByParamsForFans(db.Db, 2, msg.OperationCenterUid)
	if err != nil {
		return err
	}
	if OperationGroup == nil {
		return errors.New("当前城市合伙人暂未拥有运营中心群，请联系管理员！！！")
	} else {
		userGroup, err2 := db2.GroupUserGetOneByParams(db.ImDb, 1, map[string]interface{}{
			"key":   "group_id",
			"value": OperationGroup.GroupId,
		})
		if err2 != nil {
			return err2
		}
		if userGroup == nil {
			return errors.New("当前运营中心群暂未设置群主，请联系管理员！！！")
		}

		//加入群
		_, err = utils.GetLogicExtClient(cfg.ImLogicRpc.URL, cfg.ImLogicRpc.PORT).AddGroupMembers(utils.GetCtx("", strconv.FormatInt(userGroup.UserId, 10), "", msg.MasterId), &pb.AddGroupMembersReq{
			GroupId: int64(OperationGroup.GroupId),
			UserIds: []int64{gimUser.Id},
		})

		//发送专属红包
		gimSendSpeciallyRedPackageUser, err := db2.DbSysCfg.SysCfgGetOne("send_specially_red_package_user", msg.MasterId)
		if err != nil {
			return err
		}
		if gimSendSpeciallyRedPackageUser == nil {
			return errors.New("暂未设置专属红包发送用户")
		}
		sendSpeciallyRedPackageUser, err := db.SysCfgGetOne(db.DBs[msg.MasterId], "send_specially_red_package_user")
		if err != nil {
			return err
		}
		amount, err := getSendAmount(msg.MasterId)
		if err != nil {
			return err
		}
		args := md.SendRedPackageReq{
			UserId:                strconv.FormatInt(gimUser.Id, 10),
			DeviceId:              "",
			Token:                 "",
			Amount:                amount,
			RedPacketType:         4,
			RedPacketNums:         1,
			RedPacketContent:      "恭喜发财",
			RedPacketSmallContent: "大吉大利",
			ReceiverType:          2,
			ReceiverId:            int64(OperationGroup.GroupId),
			SendTime:              utils2.UnixMilliTime(time.Now()),
			ToUserIds:             []int64{gimUser.Id},
			RedPackageCover:       "",
		}
		err, _ = svc.CurlSendRedPackage(args, msg.MasterId, sendSpeciallyRedPackageUser.Val)
		if err != nil {
			return err
		}
	}
	return nil
}

func getSendAmount(masterId string) (string, error) {
	speciallyRedPackageAmountRadio, err := db.SysCfgGetOne(db.DBs[masterId], "specially_red_package_amount_radio")
	if err != nil {
		return "", err
	}
	var speciallyRedPackageAmountRadioData []*md.SpeciallyRedPackageAmountRadio
	err = json.Unmarshal([]byte(speciallyRedPackageAmountRadio.Val), &speciallyRedPackageAmountRadioData)
	if err != nil {
		return "", err
	}

	var tempSpeciallyRedPackageAmountRadioData []*md.SpeciallyRedPackageAmountRadio
	for _, v := range speciallyRedPackageAmountRadioData {
		for i := 0; i < utils2.StrToInt(v.Value); i++ {
			tempSpeciallyRedPackageAmountRadioData = append(tempSpeciallyRedPackageAmountRadioData, v)
		}
	}
	if len(tempSpeciallyRedPackageAmountRadioData) < 100 {
		return "", errors.New("红包金额比例设置有误！")
	}

	randInt := utils2.RandInt(99)
	min := tempSpeciallyRedPackageAmountRadioData[randInt].From
	max := tempSpeciallyRedPackageAmountRadioData[randInt].To
	amount := utils2.AnyToString(utils2.RandFloats(utils2.AnyToFloat64(min), utils2.AnyToFloat64(max)))
	return amount, nil
}
